//----------------------------------------------------------------------
//FILE dsUtil.pas
//
// Copyright (C) 1995 Microsoft Corporation. All Rights Reserved.
// Content:    Routines for dealing with sounds from resources
// Translated to Objetc Pascal by Andrs Lopehanda M.
//----------------------------------------------------------------------

unit dsUtil;
interface
uses windows,mmsystem,dsound;



//----------------------------------------------------------------------
//Constants
const c_szWAV='WAV';

      //Max Number of sound buffers
      //Aviable in TSNDOBJ
      MAX_BUFFER_COUNT=4;
      CT_TSNDOBJ_NOALLOC=-1;



//----------------------------------------------------------------------
//Structures and types
type

TSNDOBJ=record
    pbWaveData:POINTER;             // pointer into wave resource (for restore)
    cbWaveSize:DWORD;               // size of wave data (for restore)
    iAlloc:integer;                 // number of buffers.
    iCurrent:integer;               // current buffer
    Buffers:array [0..MAX_BUFFER_COUNT] of IDirectSoundBuffer; // list of buffers
end;

HSNDOBJ=^TSNDOBJ;

//----------------------------------------------------------------------
//Procedure declarations

//IDirectSoundBuffer *DSLoadSoundBuffer(IDirectSound *pDS, LPCTSTR lpName)
function DSLoadSoundBuffer(var pDS:IDirectSound;lpName:string ):IDirectSoundBuffer;


//BOOL DSReloadSoundBuffer(IDirectSoundBuffer *pDSB, LPCTSTR lpName)
function DSReloadSoundBuffer(var pDSB:IDirectSoundBuffer;lpName:string):boolean;


//BOOL DSGetWaveResource(HMODULE hModule, LPCTSTR lpName,
//    WAVEFORMATEX **ppWaveHeader, BYTE **ppbWaveData, DWORD *pcbWaveSize)
function DSGetWaveResource(Module:HMODULE;
                           lpName:string;
                           var ppWaveHeader:PWAVEFORMATEX;
                           var ppbWaveData:POINTER;
                           var pcbWaveSize:DWORD ):boolean;



//SNDOBJ *SndObjCreate(IDirectSound *pDS, LPCTSTR lpName, int iConcurrent)
function SndObjCreate(var pDS:IDirectSound;
                      lpName:string;
                      iConcurrent:integer):TSNDOBJ;


//IDirectSoundBuffer *SndObjGetFreeBuffer(SNDOBJ *pSO)
function SndObjGetFreeBuffer( pSO:TSNDOBJ):IDirectSoundBuffer;


//BOOL SndObjPlay(SNDOBJ *pSO, DWORD dwPlayFlags)
function SndObjPlay(pSO:TSNDOBJ;dwPlayFlags:DWORD ):boolean;

//BOOL SndObjStop(SNDOBJ *pSO)
function SndObjStop(pSO:TSNDOBJ):boolean;

//void SndObjDestroy(SNDOBJ *pSO)
procedure SndObjDestroy(pSO:TSNDOBJ);


//BOOL DSFillSoundBuffer(IDirectSoundBuffer *pDSB, BYTE *pbWaveData, DWORD cbWaveSize)
function DSFillSoundBuffer(var pDSB:IDirectSoundBuffer;var pbWaveData:POINTER;cbWaveSize:WORD ):boolean;







//BOOL DSParseWaveResource(void *pvRes, WAVEFORMATEX **ppWaveHeader, BYTE **ppbWaveData,DWORD *pcbWaveSize)
function DSParseWaveResource(pvRes:pointer;
                             var ppWaveHeader:PWAVEFORMATEX;
                             var ppbWaveData:POINTER;
                             var pcbWaveSize:DWORD):boolean;




//----------------------------------------------------------------------
//Implementation
implementation








//----------------------------------------------------------------------
//Procedure:
//DSLoadSoundBuffer
//Original:
//IDirectSoundBuffer *DSLoadSoundBuffer(IDirectSound *pDS, LPCTSTR lpName)
//----------------------------------------------------------------------
function DSLoadSoundBuffer(var pDS:IDirectSound;lpName:string ):IDirectSoundBuffer;
var

    pDSB:IDirectSoundBuffer;
    dsBD:DSBUFFERDESC;
    pbWaveData:POINTER;

begin
    fillchar(dsBD,sizeof(dsBD),0);

    pDSB:=nil;
    pbWaveData:=nil;

    if (DSGetWaveResource(0, lpName, dsBD.lpwfxFormat, pbWaveData, dsBD.dwBufferBytes)) then
    begin
        dsBD.dwSize := sizeof(dsBD);
        //dsBD.dwFlags := DSBCAPS_STATIC or DSBCAPS_CTRLDEFAULT or DSBCAPS_GETCURRENTPOSITION2;
        dsBD.dwFlags := DSBCAPS_STATIC + DSBCAPS_CTRLDEFAULT + DSBCAPS_GETCURRENTPOSITION2;

        if pDS.CreateSoundBuffer(dsBD,pDSB,nil)=DS_OK then
        begin
            if not DSFillSoundBuffer(pDSB, pbWaveData, dsBD.dwBufferBytes) then
            begin
                pDSB.Release();
                pDSB:=nil;
            end;
        end
        else
        begin
            pDSB:=nil;
        end;
    end;

    Result:=pDSB;
end;




///////////////////////////////////////////////////////////////////////////////
//Procedure:
//DSReloadSoundBuffer
//Original:
//BOOL DSReloadSoundBuffer(IDirectSoundBuffer *pDSB, LPCTSTR lpName)
///////////////////////////////////////////////////////////////////////////////
function DSReloadSoundBuffer(var pDSB:IDirectSoundBuffer;lpName:string):boolean;
var
    pbWaveData:POINTER;
    cbWaveSize:DWORD;
    dummy1:PWAVEFORMATEX;
begin

    result:=false;
    pbWaveData:=nil;;
    cbWaveSize:=0;

    if DSGetWaveResource(NULL, lpName,dummy1, pbWaveData, cbWaveSize) then
    begin
        if (pDSB.Restore=DS_OK) and DSFillSoundBuffer(pDSB, pbWaveData, cbWaveSize) then
	begin
	    result:= TRUE;
        end;
    end;


end;





///////////////////////////////////////////////////////////////////////////////
//Procedure:
//DSGetWaveResource
//Original:
//BOOL DSGetWaveResource(HMODULE hModule, LPCTSTR lpName,
//    WAVEFORMATEX **ppWaveHeader, BYTE **ppbWaveData, DWORD *pcbWaveSize)
///////////////////////////////////////////////////////////////////////////////

function DSGetWaveResource(Module:HMODULE;
                           lpName:string;
                           var ppWaveHeader:PWAVEFORMATEX;
                           var ppbWaveData:POINTER;
                           var pcbWaveSize:DWORD ):boolean;
var
    hResInfo:HRSRC;
    hResData:HGLOBAL;
    pvRes:pointer;

begin
    hResInfo := FindResource(Module, Pchar(lpName), c_szWAV);

    if hResInfo<>NULL then
     hResData := LoadResource(Module, hResInfo)
    else
     begin
      result:=false;
      exit;
     end;

    pvRes := LockResource(hResData);
    if (pvRes<>nil) and
       DSParseWaveResource(pvRes, ppWaveHeader, ppbWaveData, pcbWaveSize)
     then
    else
     begin
      result:=false;
      exit;
     end;

     Result:=true
end;



//----------------------------------------------------------------------
//
// SndObj functions
//
//----------------------------------------------------------------------

function SndObjCreate(var pDS:IDirectSound;
                      lpName:string;
                      iConcurrent:integer):TSNDOBJ;
var
    pSO:TSNDOBJ;
    pWaveHeader:PWAVEFORMATEX;
    pbData:POINTER;
    //cbData:UINT;
    cbData:DWORD;
    i:integer;

begin

    pSO.iAlloc :=CT_TSNDOBJ_NOALLOC;
    pWaveHeader:=nil;
    pbData:=nil;
    cbData:=0;

    //In the original source code appears a NULL instead of a 0,
    //It doesnot work here.
    if DSGetWaveResource(0,lpName,pWaveHeader,pbData,cbData) then
    begin
        if iConcurrent < 1 then iConcurrent:= 1;

        //Original line
        //pSO.iAlloc := iConcurrent;
        //Mine
        pSO.iAlloc :=MAX_BUFFER_COUNT;

        pSO.pbWaveData := pbData;
        pSO.cbWaveSize := cbData;
        pSO.Buffers[0] := DSLoadSoundBuffer(pDS, lpName);
        pSO.iCurrent   := 0;

        for i:=1 to pSO.iAlloc do
        begin
         if pDS.DuplicateSoundBuffer(pSO.Buffers[0],pSO.Buffers[i])<>DS_OK then
            begin
   	      pSO.Buffers[i] := DSLoadSoundBuffer(pDS, lpName);
  	      if pSO.Buffers[i]<>nil then
               begin
 	      	 SndObjDestroy(pSO);
                 pSO.iAlloc :=CT_TSNDOBJ_NOALLOC;
	      	exit;
               end;
            end;
        end;//for
    end;

    result:=pSO;
end;






///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//void SndObjDestroy(SNDOBJ *pSO)
procedure SndObjDestroy(pSO:TSNDOBJ);
var
  i:integer;
begin
   for i:=0 to pSO.iAlloc do
     if assigned(pSO.Buffers[i]) then
        begin
           pSO.Buffers[i].release();
           pSO.Buffers[i]:=nil;
        end;
end;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//IDirectSoundBuffer *SndObjGetFreeBuffer(SNDOBJ *pSO)
function SndObjGetFreeBuffer(pSO:TSNDOBJ):IDirectSoundBuffer;
var
    pDSB:IDirectSoundBuffer;

    hres:HRESULT;
    dwStatus:DWORD;
begin

 if pSO.iAlloc = CT_TSNDOBJ_NOALLOC then
   begin
       result:=nil;
       exit;
   end;

 pDSB:=pSO.Buffers[pSO.iCurrent];

 if assigned(pDSB) then
 begin
   hres := pDSB.GetStatus(dwStatus);

   if (FAILED(hres)) then dwStatus := 0;

   if (dwStatus and DSBSTATUS_PLAYING) = DSBSTATUS_PLAYING then
      if pSO.iAlloc > 1 then
         begin
            inc(pSO.iCurrent);
            if (pSO.iCurrent >= pSO.iAlloc) then
                pSO.iCurrent := 0;

                pDSB := pSO.Buffers[pSO.iCurrent];
                hres := pDSB.GetStatus(dwStatus);

                if SUCCEEDED(hres) and ((dwStatus and DSBSTATUS_PLAYING) = DSBSTATUS_PLAYING) then
                begin
                    pDSB.Stop();
                    pDSB.SetCurrentPosition(0);
                end;
          end
      else
         pDSB := nil;


   if (pDSB<>nil) and ((dwStatus and DSBSTATUS_BUFFERLOST)=DSBSTATUS_BUFFERLOST) then
    begin
        if FAILED(pDSB.Restore()) or (not DSFillSoundBuffer(pDSB, pSO.pbWaveData, pSO.cbWaveSize)) then
           pDSB := nil;
    end; //if
 end; //if assigned

 result:=pDSB;
end; //proc


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//BOOL SndObjPlay(SNDOBJ *pSO, DWORD dwPlayFlags)
function SndObjPlay(pSO:TSNDOBJ;dwPlayFlags:DWORD ):boolean;
var
   pDSB:IDirectSoundBuffer;
begin
    result:= FALSE;

    if pSO.iAlloc = CT_TSNDOBJ_NOALLOC then exit;

    //POR AHORA
    //if ((not (dwPlayFlags and DSBPLAY_LOOPING) or (pSO.iAlloc = 1))) then
    //begin
	pDSB:=SndObjGetFreeBuffer(pSO);
	if (pDSB <> nil) then
          result := SUCCEEDED(pDSB.Play( 0, 0, dwPlayFlags));
    //end;

end;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//BOOL SndObjStop(SNDOBJ *pSO)
function SndObjStop(pSO:TSNDOBJ):boolean;
var
  i:integer;
begin
    if pSO.iAlloc = CT_TSNDOBJ_NOALLOC then exit;

    //original
    //if (pSO == NULL)
    //    return FALSE;


    for i:=0 to pSO.iAlloc-1 do
    begin
        pSO.Buffers[i].Stop();
        pSO.Buffers[i].SetCurrentPosition(0);
    end;

    result:=TRUE;
end;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//BOOL DSFillSoundBuffer(IDirectSoundBuffer *pDSB, BYTE *pbWaveData, DWORD cbWaveSize)
function DSFillSoundBuffer(var pDSB:IDirectSoundBuffer;var pbWaveData:POINTER;cbWaveSize:WORD ):boolean;
var
    //original
    //pMem1, pMem2:LPVOID;
    //mine
    pMem1, pMem2:POINTER;
    dwSize1, dwSize2:DWORD ;
begin

    if (pDSB<>nil) and (pbWaveData<>nil) and (cbWaveSize<>NULL) then
    begin

        if (SUCCEEDED(pDSB.Lock(0, cbWaveSize,
            pMem1, dwSize1, pMem2, dwSize2, 0))) then
        begin
            CopyMemory(pMem1, pbWaveData, dwSize1);

            if ( 0 <> dwSize2 ) then
                CopyMemory(pMem2, pointer(DWORD(pbWaveData)+dwSize1), dwSize2);

            if pDSB.Unlock(pMem1, dwSize1, pMem2, dwSize2)<>DS_OK then
              begin
               result:=false;
               exit;
              end;

            result:=TRUE;
            exit;
        end;
    end;

    result:=FALSE;
end;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//BOOL DSParseWaveResource(void *pvRes, WAVEFORMATEX **ppWaveHeader, BYTE **ppbWaveData,DWORD *pcbWaveSize)
function DSParseWaveResource(pvRes:pointer;
                             var ppWaveHeader:PWAVEFORMATEX;
                             var ppbWaveData:POINTER;
                             var pcbWaveSize:DWORD):boolean;


label ex;

const
   FOURCC_WAVE = $45564157 ;   { 'WAVE' }
   FOURCC_DATA = $61746164 ;   { 'data' }
   FOURCC_FMT  = $20746d66 ;   { 'fmt ' }


   // dec      hex   letra
   // 57       87    W
   // 41       65    A
   // 86       56    V
   // 69       45    E


   // 100      64    d
   // 97       61    a                       
   // 32       20    Espacio
   // 102      66    f
   // 109      6d    m
   // 116      74    t

type
    PWORD=^DWORD;
    PBYTE=^BYTE;

var
    //DWORD *pdw;
    //DWORD *pdwEnd;
    //DWORD dwRiff;
    //DWORD dwType;
    //DWORD dwLength;

    pdw,
    pdwEnd:PWORD;
    dwRiff,
    dwType,
    dwLength:DWORD;

    temp:DWORD;
begin


    //ppWaveHeader:= NULL;
    //ppbWaveData := NULL;
    //cbWaveSize := 0;

    //pdw = pvRes;
    pdw := PWORD(pvRes);

    //dwRiff =*pdw++;
    dwRiff :=pdw^;
    inc(pdw);

    //dwLength = *pdw++;
    dwLength :=pdw^;
    inc(pdw);

    //dwType = *pdw++;
    dwType :=pdw^;
    inc(pdw);

    //C original source code.
    //mmioFOURCC is not declared in mmsystem.pas =(
    //
    //if (dwRiff <> mmioFOURCC('R', 'I', 'F', 'F'))
    //    goto exit;      // not even RIFF
    //
    //if (dwType <> mmioFOURCC('W', 'A', 'V', 'E'))
    //    goto exit;      // not a WAV
    //
    // In object Pascal you must use
    // mmioStringToFOURCC
    // mmioStringToFOURCC
    // mmioStringToFOURCC
    // mmioInstallIOProcA
    // or de constants
    // FOURCC_RIFF = $46464952;   { 'RIFF' }
    // FOURCC_LIST = $5453494C;   { 'LIST' }


    //not a riff
    if dwRiff <> FOURCC_RIFF then goto ex;

    // not a WAV
    if dwType <> FOURCC_WAVE then goto ex;



    pdwEnd := PWORD(PBYTE(integer(pdw) + integer(dwLength)-4));

    while integer(pdw) < integer(pdwEnd) do
    begin
      //dwType = *pdw++;
      dwType :=pdw^;
      inc(pdw);

      //dwLength = *pdw++;
      dwLength :=pdw^;
      inc(pdw);

      case dwType of
      FOURCC_FMT:  //'fmt '
          begin
            if ppWaveHeader=nil then
            begin
                if (dwLength < sizeof(TWAVEFORMAT)) then
                    goto ex;      // not a WAV

                ppWaveHeader:=PWAVEFORMATEX(pdw);

                if (ppbWaveData<>nil) and (pcbWaveSize<>0) then
                begin
                    Result:=true;
                    exit;
                end;
            end;
          end;

      FOURCC_DATA:
          begin
            if (ppbWaveData=nil) and (pcbWaveSize=0) then
            begin
                ppbWaveData := pdw;
                pcbWaveSize := dwLength;

                if ppWaveHeader<>nil then
                begin
                   result:=true;
                   exit;
                end;
            end;
          end;
      end; //case


        //C
        //pdw:= (DWORD *)((BYTE *)pdw + ((dwLength+1)&~1));
        //Pascal
        temp:=(dwLength+1) and (not $1);
        pdw := PWORD(PBYTE(DWORD(pdw) + temp ));

    end;

ex:
   result:=FALSE;

end; //ParseWavFile



//----------------------------------------------------------------------
end. //of unit
